#include <windows.h>
#include <stdio.h>
#include "ctx.h"

enum { wstyle = WS_OVERLAPPEDWINDOW };
enum { ratio_y = 3, ratio_x = 4 };

Ctx::Ctx()
{
	hwnd = 0; hdc = 0; hglrc = 0; hPalette = 0;
	depthon = texon = fullscreen = usersize = FALSE;
}

__declspec(dllexport) Ctx ctx;

WNDPROC oldWndProc;


void UnAdjustWindowRect(RECT* rect, DWORD style, BOOL menu)
{
	RECT rect1 = *rect;
	AdjustWindowRect(&rect1, style, menu);
	rect->top += rect->top - rect1.top;
	rect->left += rect->left - rect1.left;
	rect->right += rect->right - rect1.right;
	rect->bottom += rect->bottom - rect1.bottom;
}


void fixRatio(WPARAM edge, LPRECT prect)
{
	RECT rect = *prect;
	UnAdjustWindowRect(&rect, wstyle, FALSE);
	int width = rect.right - rect.left;
	int height = rect.bottom - rect.top;
	int dwidth = height * ratio_x / ratio_y - width;
	int dheight = width * ratio_y / ratio_x - height;
	int wh = width * ratio_y - height * ratio_x;

	switch (edge)
	{
	case WMSZ_LEFT:
	case WMSZ_RIGHT: prect->bottom += dheight; break;
	case WMSZ_TOP:
	case WMSZ_BOTTOM: prect->right += dwidth; break;
	case WMSZ_TOPLEFT: if (wh > 0) prect->top -= dheight; else prect->left -= dwidth; break;
	case WMSZ_TOPRIGHT: if (wh > 0) prect->top -= dheight; else prect->right += dwidth; break;
	case WMSZ_BOTTOMLEFT: if (wh > 0) prect->bottom += dheight; else prect->left -= dwidth; break;
	case WMSZ_BOTTOMRIGHT: if (wh > 0) prect->bottom += dheight; else prect->right += dwidth; break;
	}
}


void Ctx::toogleFullscreen()
{
	RECT rect;
	DWORD style;
	HWND after;
	if (ctx.fullscreen = !ctx.fullscreen) {
		GetWindowRect(hwnd, &ctx.winrect);
		GetWindowRect(GetDesktopWindow(), &rect);
		style = WS_POPUP;
		after = HWND_TOPMOST;
	}
	else {
		rect = ctx.winrect;
		style = wstyle;
		after = HWND_TOP;
	}
	SetWindowLong(hwnd, GWL_STYLE, style|WS_VISIBLE);
	SetWindowPos(hwnd, after, rect.left, rect.top, rect.right - rect.left, rect.bottom - rect.top,
		SWP_SHOWWINDOW|SWP_FRAMECHANGED);
}


LRESULT CALLBACK WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch (msg)
	{
	case WM_SIZE:
		return ctx.usersize = TRUE;
	case WM_SIZING:
		if (!ctx.fullscreen) fixRatio(wParam, (LPRECT) lParam);
		return TRUE;

	case WM_SYSKEYDOWN:
		if (wParam == 13 && lParam & 1 << 29)
		{
			if (!(lParam & 1 << 30)) ctx.toogleFullscreen();
			return TRUE;
		}
		return oldWndProc(hwnd, msg, wParam, lParam);

	case WM_SETCURSOR:
		if (!ctx.fullscreen) return DefWindowProc(hwnd, msg, wParam, lParam);
	case WM_KEYDOWN:
	case WM_KEYUP:
		return oldWndProc(hwnd, msg, wParam, lParam);

	case WM_CLOSE:
	case WM_DESTROY: ExitProcess(0); return 0;
	}
	return DefWindowProc(hwnd, msg, wParam, lParam);
}


static DWORD pid = 0;

BOOL CALLBACK EnumWinProc(HWND hwnd, LPARAM _this)
{
	if (!(GetWindowLong(hwnd,GWL_STYLE) & WS_VISIBLE)) return TRUE;
	DWORD pidwin;
	GetWindowThreadProcessId(hwnd, &pidwin);
	if (pidwin != pid) return TRUE;
	((Ctx*) _this)->init(hwnd);
	return FALSE;
}


void Ctx::init()
{
	if (hwnd) return;
	pid = GetCurrentProcessId();
	EnumWindows(EnumWinProc, (LPARAM) this);
}

void Ctx::init(HWND hwnd)
{
	this->hwnd = hwnd;
	SetWindowLong(hwnd, GWL_STYLE, wstyle|WS_VISIBLE);
	oldWndProc = (WNDPROC) GetWindowLongPtr(hwnd, GWL_WNDPROC);
	SetWindowLongPtr(hwnd, GWL_WNDPROC, (LONG_PTR) WndProc);
	RECT rect = { 100, 100, 740, 580 };
	AdjustWindowRect(&rect, wstyle, FALSE);
	SetWindowPos(hwnd, HWND_TOP, rect.left, rect.top, rect.right - rect.left, rect.bottom - rect.top,
		SWP_SHOWWINDOW|SWP_FRAMECHANGED);
	GetWindowRect(hwnd, &rect);

	PIXELFORMATDESCRIPTOR pfd = { sizeof(PIXELFORMATDESCRIPTOR), 1,
	PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER,
	PFD_TYPE_RGBA, 24 };
	pfd.cDepthBits = 32;
	pfd.iLayerType = PFD_MAIN_PLANE;

	hdc = GetDC(hwnd);
	int pf = ChoosePixelFormat(hdc, &pfd);
	if (!pf || !SetPixelFormat(hdc, pf, &pfd))
	{
		MessageBox(hwnd, "Can't set pixel format.", "Error", MB_OK);
		ExitProcess(1);
	}

	hglrc = wglCreateContext(hdc);
	if (!hglrc || !wglMakeCurrent(hdc, hglrc))
	{
		MessageBox(hwnd, "Can't init glcontext.", "Error", MB_OK);
		ExitProcess(1);
	}
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_GEQUAL);

	glAlphaFunc ( GL_GREATER, 0.0001 ) ;
	glEnable ( GL_ALPHA_TEST ) ;
//	glEnable (GL_BLEND);
//	glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
}


void Ctx::setSize(int width, int height)
{
	RECT rect;
	if (fullscreen || usersize)
	{
		GetClientRect(hwnd, &rect);
		int e = BitBlt(hdc, 0, 0, rect.right - rect.left, rect.bottom - rect.top,
			0, 0, 0, BLACKNESS);
	}
	else
	{
		GetWindowRect(GetDesktopWindow(), &rect);
		int dw = rect.right, dh = rect.bottom;
		GetWindowRect(hwnd, &rect);
		UnAdjustWindowRect(&rect, wstyle, FALSE);
		int x = rect.left + rect.right, y = rect.top + rect.bottom;
		RECT rect = { x - width >> 1, y - height >> 1, x + width >> 1, y + height >> 1 };
		AdjustWindowRect(&rect, wstyle, FALSE);
		int ovx = rect.right - dw, ovy = rect.bottom - dh;
		if (ovx > 0) { rect.left -= ovx; rect.right -= ovx; }
		if (ovy > 0) { rect.top -= ovy; rect.bottom -= ovy; }
		if (rect.left < 0) rect.left = 0;
		if (rect.top < 0) rect.top = 0;
		SetWindowPos(hwnd, HWND_TOP, rect.left, rect.top, rect.right - rect.left, rect.bottom - rect.top,
			SWP_SHOWWINDOW|SWP_FRAMECHANGED);
	}
}


void Ctx::viewport43()
{
	RECT rect;
	GetClientRect(hwnd, &rect);
	int w = rect.right - rect.left, h = rect.bottom - rect.top;
	int x = 0, y = 0;
	if (3*w > 4*h)
	{
		int w43 = h * 4 / 3;
		x = w - w43 >> 1;
		w = w43;
	}
	else if (3*w < 4*h)
	{
		int h43 = w * 3 / 4;
		y = h - h43 >> 1;
		h = h43;
	}
	glViewport(x, y, w - 1, h - 1);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0, width, height, 1, -1e6, 0);
}
